function antest(varargin)
% antest.m :
% estimate antenna parameters. parameters include antenna phase center offset,
% antenna phase center variation, carrier-phase multipath, code multipath,
% carrier to noise power density. inputs files shall be target antenna rinex
% observation data, reference antenna/receiver rinex observation data
% (dual-frequency), and rinex navigation message file. reference antenna must
% be placed in the vicinity of target.
% estimated antenna parameters are plotted by antplot.m.
%
% options : ([]:default)
%    file ...         rinex observation files and navigation message file
%                     first obs file is for target and second obs is for reference.
%    '-out',file      output file ['antprm.mat']
%    '-pco'           estimate only phase center postion [all]
%    '-pcv'           estimate only phase center variation/phase multipath [all]
%    '-cmp'           estimate only code multipath/snr [all]
%    '-slip'          detect slip by dual-frequency phases [no]
%    '-freq',freq     frequency (1:L1,2:L2) [1]
%    '-nmax',[n m]    spherical harmonic function order/degree [10 5]
%    '-posr',[x;y;z]  reference antenna phase center position (ecef) (m)
%                     [estimated by single point positioning]
%    '-posu',[x;y;z]  target antenna reference position (ecef) (m) [same as ref]
%    '-outc',outc     bias reset threshold of outage count [2]
%    '-minel',el      elevation cutoff angle (deg) [5]
%    '-minsn',sn      SNR/CN0 cutoff (dBHz) [0]
%    '-az',az         antenna orientation (deg) [0]
%    '-pcv',param     a priori antenna pcv and its covariances {pcv,pcvv} [none]
%                     (for stacking analysis)
%
% output mat-file :
%    t0               date of first epoh by datenum.m
%    pco              target antenna phase center offset relative to target
%                     antenna reference postion [e n u] (m)
%    pcv              estimated phase center variation parameters as
%                     coefficients of spherical harmonic function.
%    pcvv             covariance matrix of estimated pcv
%    pmp              carrier-phase multipaths (mp)  [time sat az el mp;...] (m)
%                     time is relative to t0 (s), sat is satellite prn number
%                     az, el are azimuth/elevation angle(rad)
%    cmp              code multipaths (mp) [time sat az el mp;...] (m)
%    snr              receiver snr [time sat az el snr;...] (dBHz)
%    rr               reference antenna phase center position (ecef) (m)
%    ru               target antenna reference position (ecef) (m)
%    opt              processing option struct
%
%            Copyright(c) 2007-2008 by T.Takasu, all rights reserved
%
% version: $Revision: 1.8 $ $Date: 2008/07/22 01:49:02 $
% history: 2008/06/04 1.0 new
%          2008/07/21 1.1 add slip detection by dual-frequency phases
%          2009/01/09 1.2 fix bug on error stop if reference position empty

rr=[];                % reference antenna pos (ecef)
ru=[];                % target antenna pos (ecef)
fout='antprm.mat';    % output file
type=0;
nmax=[10 5];          % degree/order of pcv spherical harmonic function
opt.f=1;              % carrier frequency (1:L1,2:L2)
opt.minel=5*pi/180;   % elevation cutoff angle (rad)
opt.minsn=35;         % minimum snr (dBHz)
opt.sig=0.01;         % measurement error of phase (m)
opt.outc=2;           % bias reset threshold of outage count
opt.slip=0;           % slip detection by geometry-free-phase jump
opt.aza=0;            % antenna azimuth offset (rad)
opt.pcv={};           % a priori phase center variation and covariance
opt.nmax=[0 0]; 

i=1; fobs={};
while i<=nargin
    switch varargin{i}
    case '-out',   i=i+1; fout=varargin{i};
    case '-pco',   type=1;
    case '-pcv',   type=2;
    case '-cmp',   type=3;
    case '-slip',  opt.slip=1;
    case '-freq',  i=i+1; opt.f=tonum(varargin{i});
    case '-nmax',  i=i+1; nmax=tonum(varargin{i});
    case '-posr',  i=i+1; rr=tonum(varargin{i});
    case '-posu',  i=i+1; ru=tonum(varargin{i});
    case '-outc',  i=i+1; opt.outc=tonum(varargin{i});
    case '-minel', i=i+1; opt.minel=tonum(varargin{i})*pi/180;
    case '-minsn', i=i+1; opt.minsn=tonum(varargin{i});
    case '-az',    i=i+1; opt.aza=tonum(varargin{i})*pi/180;
    case '-pcv',   i=i+1; opt.pcv=varargin{i};
    otherwise, fobs={fobs{:},varargin{i}};
    end
    i=i+1;
end
if length(fobs)<3
    disp('specify target and reference obs and nav file');
    return;
end
% read rinex obs/nav files
[t0,obs,iobs,nav,inav,ionp]=readrnx(fobs);
if size(obs,1)<=0, disp('no observation data'); return; end
if size(nav,1)<=0, disp('no navigation message'); return; end
i=find(1<=iobs(:,2)&iobs(:,2)<=32); obs=obs(i,:); iobs=iobs(i,:);

if isempty(rr) % if ref ant position not specified, estimate approx pos
    i=find(iobs(:,3)==2);
    rr=apppos(t0,obs(i,:),iobs(i,:),nav,inav,ionp);
    if isempty(rr), disp('reference antenna position error'); return; end
end
if isempty(ru), ru=rr; end

if opt.slip==1 % slip detection for dual-frequency phase
    [obs,iobs]=detslip(obs,iobs);
end
pco=[]; pcv=[]; pcvv=[]; pmp=[]; cmp=[]; snr=[];

if type==0|type==1 % pass-1 : estimate antenna phase center position
    opt.ru=[]; opt.rr=rr;
    [x,P,s]=antphas(t0,obs,iobs,nav,inav,ionp,opt);
    [p,E]=togeod(ru); pco=E*(x(1:3)-ru);
end
if type==0|type<=2 % pass-2 : estimate antenna pcv and phase multipath
    opt.nmax=nmax; opt.ru=x(1:3); opt.rr=rr;
    [x,P,s]=antphas(t0,obs,iobs,nav,inav,ionp,opt);
    n=3+32+1; pcv=x(n:end); pcvv=P(n:end,n:end); pmp=s;
end
if type==0|type==3 % pass-3 : estimate code multipath and snr
    [cmp,snr]=antcode(t0,obs,iobs,nav,inav,ru,opt);
end
% save estimated parameter
disp(['writing output file ... : ',fout]);
save(fout,'t0','pco','pcv','pcvv','pmp','cmp','snr','rr','ru','opt');

% approximate position ---------------------------------------------------------
function rr=apppos(t0,obs,iobs,nav,inav,ionp)
nav=[nav,(datenum(1980,1,6)+nav(:,28)*7-t0)*86400+nav(:,18)];
[time,i]=unique(round(iobs(:,1))); i=[1;i+1]; rr=[];
for j=1:length(time)
    if mod(time(j),300), continue; end
    k=i(j):i(j+1)-1; [r,t,Q]=pntpos(obs(k,:),iobs(k,:),nav,inav,ionp);
    if trace(Q)<=3^2, rr=[rr,r]; end
end
if ~isempty(rr)
    i=find(~isnan(rr(1,:))); if ~isempty(i), rr=mean(rr(:,i),2); else rr=[]; end
end

% detect slip by geometry-free-phase jump --------------------------------------
function [obs,iobs]=detslip(obs,iobs)
C=299792458; f=[1.57542E9;1.2276E9];
gfp=repmat(nan,32,2);
[time,i]=unique(round(iobs(:,1))); i=[1;i+1];
for j=1:length(time)
    k=i(j):i(j+1)-1;
    for n=1:2
        for m=find(iobs(k,3)==n)'
            sat=iobs(k(m),2);
            gf=C*(obs(k(m),1)/f(1)-obs(k(m),2)/f(2));
            if abs(gf-gfp(sat,n))>0.05
                iobs(k(m),4)=bitor(iobs(k(m),4),1);
                iobs(k(m),5)=bitor(iobs(k(m),5),1);
                disp(sprintf('TIME=%6.0f : SLIP PRN%2d %6.3fm',iobs(k(m(1)),1),...
                     sat,gf-gfp(sat,n)));
            elseif isnan(gf), obs(k(m),1:2)=nan; end
            gfp(sat,n)=gf;
        end
    end
end

% number argument --------------------------------------------------------------
function n=tonum(s), if isstr(s), n=str2num(s); else n=s; end
